%    Eotvos Quantum Transport Utilities - Frequently asked questions.
%    Copyright (C) 2018 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.

%>   \page faq Frequently asked questions.
%> 	Here are discussed frequently asked quetions related to the usage of EQuUs
%>
%>  ## 1. Offloading EQuUs MKL interface to Intel MIC architectures
%>   
%>   ### Description
%>   When compiling MEX files for MATLAB 2015b (or less) with Intel MIC support one gets the following error during runtime:
%>     Invalid MEX-file '/path/to/MEXfile/dgetPartialInv.mexa64': /opt/intel/compilers_and_libraries_2016.1.150/linux/compiler/lib/intel64/liboffload.so.5: symbol __kmpc_proxy_task_completed_ooo, version VERSION not defined in file libiomp5.so with link time reference
 %>  ### Solution
%>   The libiomp5.so library shipped with MATLAB 2015b (or less) is incompatible with the MEX files compiled with MIC support. Possible workaround is to replace the libiomp5.so library located at "MATLABROOT/sys/os/glnxa64/" with the library provided by the Intel compiler usually located at /opt/intel/lib/intel64/libiomp5.so.
%>  
%>  
%>  
%>  ## 2. Running deployed MATLAB applications on NIIF Debrecen2 cluster with Matlab Runtime Environment v90
%>  
%>   ### Description
%>   When running deployed MATLAB applications on Debrecen2 cluster with Matlab Runtime Environment v90 one gets the following error message:
%>   An Error has occurred while trying to initialize the MATLAB Runtime.
%>   The error is: Fatal error loading library /Lustre01/home/rakytap/source/MCR/v90/bin/glnxa64/libmwmclmcr.so Error: libXt.so.6: cannot open shared object file: No such file or directory
%>   Error:mclmcr initialization failed
%>  
%>   ### Solution
%>   Filesystem on the debrecen2 cluster does not contains the dependencies of the library libmwmclmcr.so. Thus one must provide these dependencies for the MCR at runtime. One can collect the dependencies of the shared library libmwmclmcr.so into /path/to/dependecies by, for example, a bash tool archived in <a href="http://eqt.elte.hu/EQuUs/collect_libraries.tar">collect_libraries.tar</a>. %>  Then edit the run_program.sh file generated by the MATLAB compiler for the standalone binary called "program". Add the following line to the file before "export LD_LIBRARY_PATH;": </br>
%>   LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:/path/to/dependecies;
%>   
%>   
%>   
%>   ## 3. Running deployed MATLAB applications with Matlab Parallel Toolbox support on NIIF Debrecen2 cluster with Matlab Runtime Environment
%>   
%>   ### Description
%>   When running deployed MATLAB applications on Debrecen2 cluster with Matlab Runtime Environment v90 one gets the following error message: 
%>   Failed to start a parallel pool. Can't reload '/Lustre01/home/rakytap/source/MCR/v901/bin/glnxa64/libmwhgbuiltins.so'
%>   ### Solution
%>   Filesystem on the debrecen2 cluster does not contains the dependencies of the library libmwhgbuiltins.so. Thus one must provide these dependencies for the MCR at runtime. One can collect the
%>  dependencies of the shared library libmwhgbuiltins.so into /path/to/dependecies_POOL by, for example, a bash tool archived in <a href="http://eqt.elte.hu/EQuUs/
%> collect_libraries.tar">collect_libraries.tar</a>. Then edit the run_program.sh file generated by the MATLAB compiler for the standalone binary called "program". Add the following line to the file 
%>  before "export LD_LIBRARY_PATH;": </br>
%>   LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:/path/to/dependecies_POOL;
%>   
