%%   ScatterPotential
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @file ScatterPotential.m
%> @brief Function for island shaped hard-wall potential profile. Example taken from Eur. Phys. J. B 53, 537-549 (2006).
%
%> @brief Function for island shaped hard-wall potential profile. Example taken from Eur. Phys. J. B 53, 537-549 (2006).
%> @param coordinates An instance of structure #coordinates.
%> @param island An instance of structure #circle.
%> @return An N x 1 array of the on-site potential, where N is the number of sites in input coordinates. 
    function ret = ScatterPotential( coordinates, island )
        
        island_center = [ island.center.x, island.center.y ];
        island_radius = island.radius;
        
        x = coordinates.x;
        y = coordinates.y;
        
        indexes = ( (x - island_center(1)).^2 + ( y - island_center(2)).^2 ) <= island_radius^2;
        
        ret = zeros( size(x) );
        ret(indexes) = 1;
    end
