%    Calculates the conductance for specific transverse momentums - based on EQuUs v4.8
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @file CalculateTransporSpecq.m
%> @brief Function to calculate the DC transverse momentum resolved conductivity in the zero temperature limit.
%
%
%> @brief Function to calculate the DC transverse momentum resolved conductivity in the zero temperature limit.
%> @param qvec The transverse momentum number.
%> @param width_uc The width of the unit cell.
%> @param height The length of the junction
%> @param Energy The energy relative to the Fermi level
%> @param EF The Fermi level
%> @param Opt An instance of structure #Opt
%> @param param An instance of structure #param
%> @param outputXML The absolute path to the output XML
%> @param FiniteGreensFunctionFromHamiltonian Logical value. Set true to calculate the conductivity using EQuUs MKL component, or false to use the fast method (see PRB 90, 125428 (2014)).
%> @return Returns with the calculated conductivity for the given transverse momentum number.
    function Cq = CalculateTransporSpecq( qvec, width_uc, height, Energy, EF, Opt, param, outputXML, FiniteGreensFunctionFromHamiltonian )
        Cq = zeros(size(qvec));
        
        for idx = 1:length(qvec)       
            q = qvec(idx);
            ribbon_hamiltonian = Ribbon('width', width_uc, 'height', height, 'E', Energy, 'EF', EF, 'Opt', Opt, 'param', param, 'filenameOut', outputXML, 'q', q, 'silent', true ); 
            
            [~,~,C_tmp] = ribbon_hamiltonian.Transport( Energy, 'constant_channels',true, 'gfininvfromHamiltonian', FiniteGreensFunctionFromHamiltonian );
            Cq(idx) = C_tmp;
        end
        
    end