%%    Eotvos Quantum Transport Utilities - structures
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    This function creates a data structure used by EQuUs. (since EQuUs v4.5)
%%
function ret = structures( name )

if strcmp( name, 'hole')
	%> Structure hole contains the data about the antidot used in class antidot
	ret = struct( ... name: hole
	... %> An instance of structure @coordinates containing the coordinates of the center. 
	'center', [], ... type: coordinates
	... %> The radius of the hole in the units of the lattice constant. 
	'radius',[] ...
	);   

	ret.center = structures('coordinates');
    
elseif strcmp( name, 'circle')
	%> Structure circle contains data describing a circle shaped area positioned in a two-dimensional space
	ret = struct( ... name: circle
	... %> An instance of structure @coordinates containing the coordinates of the center. 
	'center', [], ... type: coordinates
	... %> The radius of the circle . 
	'radius',[] ...
	);   

	ret.center = structures('coordinates');    

elseif strcmp( name, 'param')
	%> Structure param contains data structures describing the physical parameters of the scattering center and the leads.
	ret = struct( ... name: param
	... %> An instance of the structure @scatter_param containing the physical parameters for the scattering region. 
	'scatter',[], ...type: scatter_param
	... %> A list of structures @lead_param containing the physical parameters for the scattering region. 
	'Leads',[] ... type: lead_param
	);

elseif strcmp( name, 'scatter_param') %NEW vargamma2
	%> Structure scatter_param contains the physical parameters describing the scattering region.
	ret = struct( ... name: scatter_param
	... %> A physical parameter, see the individual lattice documentations for details
	'epsilon',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma',[],...
	... %> A physical parameter, see the individual lattice documentations for details
	'SOintrinsic',[],...
	... %> A physical parameter, see the individual lattice documentations for details
	'deltaAB',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'SO_rashba_intrinsic',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'pair_potential',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma1', [], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma2',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma3', [], ...
	... %> An instance of structure @shape describing the geometry of the scattering region. 
	'shape',[], ... type: shape
	... %> A physical parameter, see the individual lattice documentations for details
	'End_Type',[], ...
	... %> String containing the filename of the external Hamiltonain source  (see Custom_Hamiltonians for details) 
	'FileName', [], ...
	... %> Cell array of structure @Atom. 
	'Atom', [] ... type: Atom
	);

elseif strcmp( name, 'lead_param') %NEW vargamma2
	%> Structure containing the physical parameters describing a given lead.
	ret = struct( ... name: lead_param
	... %> A physical parameter, see the individual lattice documentations for details
	'epsilon',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'SOintrinsic',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'deltaAB',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'SO_rashba_intrinsic',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'pair_potential',[], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma1', [], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma2', [], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma3', [], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'vargamma_sc', [], ...
	... %> Orientation of the lead. Set +1 (default) is the "incoming" direction of the propagating states is defined in the +x or +y direction, and "-1" otherwise.
	'Lead_Orientation',[], ...
	... %> The number of regular sites in the cross section of one unit cell in the lead.
	'M', [], ...
	... %> A physical parameter, see the individual lattice documentations for details
	'End_Type',[], ...
	... %> String containing the filename of the external Hamiltonain source  (see Custom_Hamiltonians for details) 
	'FileName', [], ...
	... %> Cell array of structure @Atom. 
	'Atom', [] ... type: Atom
	);

elseif strcmp( name, 'Opt')
	%> Structure Opt contains the basic computational parameters used in EQuUs
	ret = struct( ... name: Opt
	... %> Set 1 in order to supress output messages. 
	'Silent', true,... 
	... %> Set 1 to use the Bogoliubov de Gennes model, 0 (default) for normal systems.
	'BdG',[],...
	... %> Obsolete parameter
	'Just_Create_Hamiltonians',[],...
	... %> Set true if a magnetic field is involved in the calculations, false (default) otherwise. 
	'magnetic_field',[],...
	... %> Set true (default) if a magnetic field is translational invariant along the scattering center, or false otherwise. 
	'magnetic_field_trans_invariant',[],...    
    ... %> String containining a bult-in vector potential. Possible values are: 'LandauX', 'LandauY'
	'gauge',[],...  
    ... %> Scalar value representing the strength of the bult-in vector potential: \f$ \eta = ea^2B/\hbar \f$ with \f$a\f$ standing for the atom-atom distance.
	'eta',[],...      
	... %> Set 1 to use linear interpolation for the vector potential between the atomic sites to calculate the Peierls phases. Useful when dealing with a homogeneous magnetic field. 
	'Linear_Regression_in_B',[],...
	... %> Set 1 if a simple analytical surface greens function computational method is about to be used, 0 otherwise (Only for square lattice without magnetic field). 
	'Simple_Green_Function',[],...
	... %> String describing the preprogrammed lattice type. See the documentation of lattice types for details
	'Lattice_Type',[], ...
	... %> Number of leads attached to the scattering region. 
	'NofLeads',[],...
	... %> Obsolete parameter
	'Decimate_only_Dyson',[],...
	... %> The maximal number of the sites to be decimated at once. 
	'Decimation_block',[],...
	... %> Option for using decimation algorith. (see more details at the documenatation of class Decimation) 
	'Decimation',[],...
	... %> Set 1 to use dual modes in the calculations, or 0 to use the left and right sided eigenvectors instead. 
	'usingDualModes',[], ... 
	... %> Set 1 to export debug informations into the debug.txt file, or 0 otherwise. 
	'debug', [],...
	... %> Number of workers in the parallel pool. 
	'workers', [], ...
	... %> Set 'siesta' to import Hamiltonians from Siesta, 'custom' to use custom defined Hamiltonians. 
	'custom_Hamiltonians', [],...
	... %> Set 1 to use 1/2 spin degree of freedom in the Hamiltonians, or 0 otherwise. 
	'Spin', []...
	); 

elseif strcmp( name, 'coordinates')
	ret = Coordinates();

elseif strcmp( name, 'shape')
	%> Structure shape contains data about the geometry of the scattering region.
	ret = struct( ... name: shape
	... %> The number of sites in the cross section. 
	'width',[],...
	... %> Number of unit cells along the logitudinal direction in the scattering region. 
	'height',[] ...
	);

elseif strcmp( name, 'scatterers')
	%> Structure scatterers contains data on the scattering impurities used in class antidot
	ret = struct( ... name: scatterers
	... %>  A vector of slab indexes in the ribbon. 
	'z',[],...
	... %>  A vector of site indexes in the slabs. 
	'zpoints',[],...
	... %>  A vector of the on-site potentials. 
	'potentials',[],...
	... %>  A vector of the site indexes
	'siteindexes',[],...
	... %>  Set true if the given sites are about to remove from the lattice. 
	'aremissing',[]...
	);

elseif strcmp( name, 'BandWidth')
	%> Structure BandWidth describes the bandwidth in the lead and in the scattering center.
	ret = struct( ... name: BandWidth
	... %> An instance of the structure @BandWidthLimits for the lead. 
	'Lead',[],...
	... %>  An instance of the structure @BandWidthLimits for the scattering region. 
	'Scatter',[]...
	);

elseif strcmp( name, 'BandWidthLimits')
	%> Structure BandWidthLimits contains the bandwidth limits.
	ret = struct( ... name: BandWidthLimits
	... %>  The minimal value of the energy band. 
	'Emin',[],...
	... %>  The maximal value of the energy band. 
	'Emax',[]...
	);
    
elseif strcmp( name, 'Atom') 
	%> Structure Atom contains the atomic identifiers of the sites.
	ret = struct( ... name: Atom
	... %>  Identification number of the principal layer containing the atom. 
	'PL', [],... 
	... %>  Identification number of the atom. 
	'Id', []...
	);  
	ret.PL = cell(0);
	ret.Id = cell(0);  
    
elseif strcmp( name, 'open_channels')
	%> Structure open_channels describes the open channel in the individual leads.
	ret = struct( ... name: open_channels
	... %>  Number of open channels. 
	'num', [],...
	... %>  Logical indexes of incoming electron-like channels. 
	'BdG_u_incoming', [],...
	... %>  Logical indexes of outgoing electron-like channels. 
	'BdG_u_outgoing', [],...
	... %>  true of the system is superconducting, false otherwise. 
	'isSuperconducting', [],...
	... %>  Logical indexes of ingoing channels. 
	'open_channels_p', [],...
	... %>  Logical indexes of outgoing channels. 
	'open_channels_m', []...
	);  
    
elseif strcmp( name, 'junction_sites')
	%> Structure junction_sites contains data to identify the individual sites of the leads, the interface regions and the scattering region in a matrix.
	ret = struct( ... name: junction_sites
	... %>  Structure @sites describing the geometry of the leads. 
	'Leads', [],... type: sites
	... %>  Structure @sites describing the geometry of the scattering region. 
	'Scatter', [],... type: sites
	... %>  Structure @sites describing the geometry of the interface regions. 
	'Interface', []... type: sites
	);   
    
elseif strcmp( name, 'sites')
	%> Structure sites contains data to identify the individual sites in a matrix.
	ret = struct( ... name: sites
	... %>  An instance of structure @coordinates describing the geometry. 
	'coordinates', [],...
	... %> An array containing the site indexes of the given system part. 
	'site_indexes', []...
	);    
    
elseif strcmp( name, 'DOS') %NEW
	%> Structure containg the energy resolved density of states
	ret = struct( ... name: DOS
	... %> Array of the density of states values
	'DOSvec', [],...
	... %> Array of the enrgy points
	'Evec', [] ...
	);        

elseif strcmp( name, 'WaveFnc') 
	%> Structure containg datat on the calculated eigenstate in a translational invariant lead.
	ret = struct( ... name: WaveFnc
    ... %> Cell array containing the individual wave functions.
          'WaveFnc', [], ...
    ... %> Cell array containing the individual k-pints.
          'ka', cell(length(ka_vec),1), ...
    ... %> Cell array containing the individual energies.
            'E', cell(length(ka_vec),1) ...
            );
    
else
    err = MException('structure:name', 'Unrecognized structure name');
    save('structure.mat');
    throw(err); 
end



end





